library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use ieee.math_real.all;
-- https://statics.cirrus.com/pubs/proDatasheet/CS5343-44_F5.pdf

entity ADC_CS5343 is port(
	SDOUT: out std_logic;
	SCLK: in std_logic;
	LRCK: in std_logic;
	MCLK: in std_logic;
	L: out std_logic_vector(23 downto 0);
	R: out std_logic_vector(23 downto 0));
end ADC_CS5343;

architecture Behavioral of ADC_CS5343 is

signal LRCK_buff: std_logic:='0';
signal sr: std_logic_vector(23 downto 0):=(others => '0');

type rom1kx24 is array (0 to 1023) of signed(23 downto 0); 
signal sinus_rom: rom1kx24;

signal adr: unsigned(9 downto 0):=(others => '0');
signal samplecounter: integer range 0 to 127:=0;
signal speed: unsigned(8 downto 0):="000001000";

begin

tabel : for I in 0 to 1023 generate
	sinus_rom(I) <= to_signed(integer(sin(2.0*MATH_PI*(real(I)+0.5)/1024.0)*8388607.5),24);
end generate;

process begin
	wait until falling_edge(SCLK);
	LRCK_buff <= LRCK;
	if LRCK_buff = '0' and LRCK = '1' then
		sr <= std_logic_vector(sinus_rom(to_integer(adr)));
		L <= std_logic_vector(sinus_rom(to_integer(adr)));
	elsif LRCK_buff = '1' and LRCK = '0' then
		sr <= std_logic_vector(sinus_rom(to_integer(adr)));
		R <= std_logic_vector(sinus_rom(to_integer(adr)));
		
		if samplecounter < 127 then
			samplecounter <= samplecounter +1;
		else
			samplecounter <= 0;
			if speed < 500 then
				speed <= speed +5;
			end if;
		end if;
		adr <= adr + speed;
	else
		sr <= sr(22 downto 0)  & '0';
		SDOUT <= sr(23);
	end if;
end process;

end Behavioral;