library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use std.textio.all;

entity tb_AudioInterface is
end tb_AudioInterface;

architecture Behavioral of tb_AudioInterface is

component AudioInterface is port(
	sysclk_p: in std_logic;
	sysclk_n: in std_logic;
	FT232R_TX: out std_logic;
	FT232R_RX: in std_logic;
	DA_LRCK: OUT STD_LOGIC;
	DA_MCLK: OUT STD_LOGIC;
	DA_SDIN: OUT STD_LOGIC;
	DA_SCLK: OUT STD_LOGIC;
	AD_LRCK: OUT STD_LOGIC;
	AD_MCLK: OUT STD_LOGIC;
	AD_SDOUT: IN STD_LOGIC;
	AD_SCLK: OUT STD_LOGIC);
end component;

component ADC_CS5343 is port(
	SDOUT: out std_logic;
	SCLK: in std_logic;
	LRCK: in std_logic;
	MCLK: in std_logic;
	L: out std_logic_vector(23 downto 0);
	R: out std_logic_vector(23 downto 0));
end component;

component DAC_CS4344 is port(
	SDIN: in std_logic;
	SCLK: in std_logic;
	LRCK: in std_logic;
	MCLK: in std_logic;
	L: out std_logic_vector(23 downto 0);
	R: out std_logic_vector(23 downto 0));
end component;

component UART is
    Generic(
	clk_freq: integer;
	baudrate: integer);
    Port(
	clk: in STD_LOGIC;
	start_snd: in std_logic;
	start_rcv: out std_logic;
	byte_snd: in std_logic_vector(7 downto 0);
	byte_rcv: out std_logic_vector(7 downto 0);
	tx: out std_logic;
	rx: in std_logic;
	ready: out std_logic);
end component;

signal In_Left: std_logic_vector(23 downto 0):=(others => '0');
signal In_Right: std_logic_vector(23 downto 0):=(others => '0');
signal Out_Left: std_logic_vector(23 downto 0):=(others => '0');
signal Out_Right: std_logic_vector(23 downto 0):=(others => '0');

signal clk200: std_logic:='0';
signal sysclk_p: std_logic:='0';
signal sysclk_n: std_logic:='0';

signal FT232R_TX: std_logic:='0';
signal FT232R_RX: std_logic:='0';
signal FT232R_start_snd: std_logic:='0';
signal FT232R_start_rcv: std_logic:='0';
signal FT232R_byte_snd: std_logic_vector(7 downto 0):="00000000";
signal FT232R_byte_rcv: std_logic_vector(7 downto 0):="00000000";
signal FT232R_ready: std_logic:='0';

signal DA_LRCK: std_logic:='0';
signal DA_MCLK: std_logic:='0';
signal DA_SDIN: std_logic:='0';
signal DA_SCLK: std_logic:='0';

signal AD_LRCK: std_logic:='0';
signal AD_MCLK: std_logic:='0';
signal AD_SDOUT: std_logic:='0';
signal AD_SCLK: std_logic:='0';

type fsm_type is(s_wait, s_rcv, s_check, s_set);
signal fsm: fsm_type:= s_wait;
type Packet_Array is array (0 to 13) of std_logic_vector(7 downto 0); 
signal Packet: Packet_Array:=(others => x"00");
signal Packet_valid: std_logic_vector(0 to 13):=(others => '0');

signal sync: std_logic:='0';
signal A_sign: std_logic:='0';
signal B_sign: std_logic:='0';
signal C_sign: std_logic:='0';
signal D_sign: std_logic:='0';
signal A_bits: std_logic_vector(22 downto 0):=(others => '0');
signal B_bits: std_logic_vector(22 downto 0):=(others => '0');
signal C_bits: std_logic_vector(22 downto 0):=(others => '0');
signal D_bits: std_logic_vector(22 downto 0):=(others => '0');
signal A: integer range -2**23 to 2**23-1:=0;
signal B: integer range -2**23 to 2**23-1:=0;
signal C: integer range -2**23 to 2**23-1:=0;
signal D: integer range -2**23 to 2**23-1:=0;

file file_handler: text open write_mode is "UART.txt";

begin

-- Format:
-- 0AAAAAA1
-- AAAAAAA0
-- AAAAAAA0
-- AAAABBB0
-- BBBBBBB0
-- BBBBBBB0
-- BBBBBBB0
-- 0CCCCCC0
-- CCCCCCC0
-- CCCCCCC0
-- CCCCDDD0
-- DDDDDDD0
-- DDDDDDD0
-- DDDDDDD0

process
variable row: line;
begin
	wait until rising_edge(clk200);
	if fsm = s_wait then
		if FT232R_start_rcv = '1' then
			Packet <= Packet(1 to 13) & FT232R_byte_rcv;
			Packet_valid <= Packet_valid(1 to 13) & '1';
			fsm <= s_rcv;
		end if;
	elsif fsm = s_rcv then
		if Packet_valid = "11111111111111" then
			fsm <= s_check;
		else
			fsm <= s_wait;
		end if;
	elsif fsm = s_check then
		if 	Packet(0)(0) = '1' and Packet(1)(0) = '0' and Packet(2)(0) = '0' and
			Packet(3)(0) = '0' and Packet(4)(0) = '0' and Packet(5)(0) = '0' and
			Packet(6)(0) = '0' and Packet(7)(0) = '0' and Packet(8)(0) = '0' and
			Packet(9)(0) = '0' and Packet(10)(0) = '0' and Packet(11)(0) = '0' and
			Packet(12)(0) = '0' and Packet(13)(0) = '0' then
			sync <= '1';
			A_sign <= Packet(0)(6);
			A_bits <= Packet(0)(5 downto 1) & Packet(1)(7 downto 1) & Packet(2)(7 downto 1) & Packet(3)(7 downto 4);
			B_sign <= Packet(3)(3);
			B_bits <= Packet(3)(2 downto 1) & Packet(4)(7 downto 1) & Packet(5)(7 downto 1) & Packet(6)(7 downto 1);
			C_sign <= Packet(7)(6);
			C_bits <= Packet(7)(5 downto 1) & Packet(8)(7 downto 1) & Packet(9)(7 downto 1) & Packet(10)(7 downto 4);
			D_sign <= Packet(10)(3);
			D_bits <= Packet(10)(2 downto 1) & Packet(11)(7 downto 1) & Packet(12)(7 downto 1) & Packet(13)(7 downto 1);
			Packet_valid <= (others => '0');
			fsm <= s_set;
		else
			if sync = '1' then
				report "Fehler";
			end if;
			sync <= '0';
			fsm <= s_wait;
		end if;
	elsif fsm = s_set then
		if A_sign = '0' then
			A <= to_integer(unsigned(A_bits(22 downto 0)));
		else
			A <= -1*(2**23-to_integer(unsigned(A_bits(22 downto 0))));
		end if;
		if B_sign = '0' then
			B <= to_integer(unsigned(B_bits(22 downto 0)));
		else
			B <= -1*(2**23-to_integer(unsigned(B_bits(22 downto 0))));
		end if;
		if C_sign = '0' then
			C <= to_integer(unsigned(C_bits(22 downto 0)));
		else
			C <= -1*(2**23-to_integer(unsigned(C_bits(22 downto 0))));
		end if;
		if D_sign = '0' then
			D <= to_integer(unsigned(D_bits(22 downto 0)));
		else
			D <= -1*(2**23-to_integer(unsigned(D_bits(22 downto 0))));
		end if;
		write(row, integer'image(A));
		write(row, string'(", "));
		write(row, integer'image(B));
		write(row, string'(", "));
		write(row, integer'image(C));
		write(row, string'(", "));
		write(row, integer'image(D));
		writeline(file_handler, row);
		fsm <= s_wait;
	end if;
end process;

clk200 <= not clk200 after 2.5 ns;
sysclk_p <= clk200;
sysclk_n <= not clk200;

uut: AudioInterface port map(
	sysclk_p => sysclk_p,
	sysclk_n => sysclk_n,
	FT232R_TX => FT232R_TX,
	FT232R_RX => FT232R_RX,
	DA_LRCK => DA_LRCK,
	DA_MCLK => DA_MCLK,
	DA_SDIN => DA_SDIN,
	DA_SCLK => DA_SCLK,
	AD_LRCK => AD_LRCK,
	AD_MCLK => AD_MCLK,
	AD_SDOUT => AD_SDOUT,
	AD_SCLK => AD_SCLK);
	
adc: ADC_CS5343 port map(
	SDOUT => AD_SDOUT,
	SCLK => AD_SCLK,
	LRCK => AD_LRCK,
	MCLK => AD_MCLK,
	L => In_Left,
	R => In_Right);
	
dac: DAC_CS4344 port map(
	SDIN => DA_SDIN,
	SCLK => DA_SCLK,
	LRCK => DA_LRCK,
	MCLK => DA_MCLK,
	L => Out_Left,
	R => Out_Right);
	
uart_bench: UART
    Generic map(
	clk_freq => 200000000,
	baudrate => 8000000)
    Port map(
	clk => clk200,
	start_snd => FT232R_start_snd,
	start_rcv => FT232R_start_rcv,
	byte_snd => FT232R_byte_snd,
	byte_rcv => FT232R_byte_rcv,
	tx => FT232R_RX,
	rx => FT232R_TX,
	ready => FT232R_ready);

end Behavioral;