library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity AudioInterface is port(
    sysclk_p    : in std_logic;
    sysclk_n    : in std_logic;
	--- UART ---
	FT232R_TX	: out std_logic;
	FT232R_RX	: in std_logic;
--*************************alle PMODS in Entity********************************************
    DA_LRCK     : OUT STD_LOGIC;
    DA_MCLK     : OUT STD_LOGIC;
    DA_SDIN     : OUT STD_LOGIC;
    DA_SCLK     : OUT STD_LOGIC;
    AD_LRCK     : OUT STD_LOGIC;
    AD_MCLK     : OUT STD_LOGIC;
    AD_SDOUT    : IN STD_LOGIC;
    AD_SCLK     : OUT STD_LOGIC);
--*****************************************************************************************
end AudioInterface;

architecture Behavioral of AudioInterface is

component UART is
    Generic(
	clk_freq: integer;
	baudrate: integer);
    Port(
	clk: in STD_LOGIC;
	start_snd: in std_logic;
	start_rcv: out std_logic;
	byte_snd: in std_logic_vector(7 downto 0);
	byte_rcv: out std_logic_vector(7 downto 0);
	tx: out std_logic;
	rx: in std_logic;
	ready: out std_logic);
end component;

component i2s_data_interface Port( 
	clk           : in  STD_LOGIC;
	audio_l_in    : in  STD_LOGIC_VECTOR (23 downto 0);
	audio_r_in    : in  STD_LOGIC_VECTOR (23 downto 0);
	audio_l_out   : out STD_LOGIC_VECTOR (23 downto 0);
	audio_r_out   : out STD_LOGIC_VECTOR (23 downto 0);
	new_sample    : out STD_LOGIC;
	i2s_bclk      : in  STD_LOGIC;
	i2s_d_out     : out STD_LOGIC;
	i2s_d_in      : in  STD_LOGIC;
	i2s_lr        : in  STD_LOGIC);
end component;

component CLK_I2S port(
	clk200: out std_logic;
	clk8_192: out std_logic;
	clk_in1_p: in std_logic;
	clk_in1_n: in std_logic);
end component;

component FIR_L port(
	aclk : IN STD_LOGIC;
	s_axis_data_tvalid : IN STD_LOGIC;
	s_axis_data_tready : OUT STD_LOGIC;
	s_axis_data_tdata : IN STD_LOGIC_VECTOR(23 DOWNTO 0);
	m_axis_data_tvalid : OUT STD_LOGIC;
	m_axis_data_tdata : OUT STD_LOGIC_VECTOR(23 DOWNTO 0));
end component;

component FIR_R port(
	aclk : IN STD_LOGIC;
	s_axis_data_tvalid : IN STD_LOGIC;
	s_axis_data_tready : OUT STD_LOGIC;
	s_axis_data_tdata : IN STD_LOGIC_VECTOR(23 DOWNTO 0);
	m_axis_data_tvalid : OUT STD_LOGIC;
	m_axis_data_tdata : OUT STD_LOGIC_VECTOR(23 DOWNTO 0));
end component;

-------------------------------------------
signal FT232R_start_snd: std_logic:='0';
signal FT232R_start_rcv: std_logic:='0';
signal FT232R_byte_snd: std_logic_vector(7 downto 0):="00000000";
signal FT232R_byte_rcv: std_logic_vector(7 downto 0):="00000000";
signal FT232R_ready: std_logic:='0';
type fsm_type is(s_wait, s_send);
signal fsm: fsm_type:= s_wait;
signal packet_bytecounter: integer range 0 to 13:=0;
signal packet: std_logic_vector(8*14-1 downto 0):=(others => '0');
---------------------------------------
signal s_filter_input_data_l_16_bit : std_logic_vector(15 downto 0):=(others => '0');
signal s_filter_input_data_r_16_bit : std_logic_vector(15 downto 0):=(others => '0');
signal s_filtered_output_data_l_16_bit : std_logic_vector(15 downto 0):=(others => '0');
signal s_filtered_output_data_r_16_bit : std_logic_vector(15 downto 0):=(others => '0');
------------------------------------------------------
signal s_reset              : std_logic:='0';
signal s_clk_200_MHz        : std_logic:='0';
signal s_clk_8_192_MHz_MCLK : std_logic:='0';
signal s_clk_2_048_MHz_SCLK : std_logic:='0';
signal s_clk_32_kHz_LRCK    : std_logic:='0';
signal clkdiv: unsigned(7 downto 0):=(others => '0');
------------------------------------------------------
signal s_audio_l_in : std_logic_vector(23 downto 0):=(others => '0');
signal s_audio_r_in : std_logic_vector(23 downto 0):=(others => '0');
------------------------------------------------------
signal s_audio_l_out : std_logic_vector(23 downto 0):=(others => '0');
signal s_audio_r_out : std_logic_vector(23 downto 0):=(others => '0');
------------------------------------------------------
signal s_new_sample: std_logic:='0';
signal s_new_sample_old: std_logic:='0';
signal s_DA_SDIN: std_logic:='0';
signal s_valid: std_logic :='0';

begin

process begin
	wait until rising_edge(s_clk_200_MHz);
	s_new_sample_old <= s_new_sample;
	FT232R_start_snd <= '0';
	if fsm = s_wait then
		if s_new_sample = '1' and s_new_sample_old = '0' then
			packet_bytecounter <= 0;
			-- Format:
			-- 0AAAAAA1
			-- AAAAAAA0
			-- AAAAAAA0
			-- AAAABBB0
			-- BBBBBBB0
			-- BBBBBBB0
			-- BBBBBBB0
			-- 0CCCCCC0
			-- CCCCCCC0
			-- CCCCCCC0
			-- CCCCDDD0
			-- DDDDDDD0
			-- DDDDDDD0
			-- DDDDDDD0
			packet <=   '0' & s_audio_l_out(23 downto 18) & '1'
                & s_audio_l_out(17 downto 11) & '0'
                & s_audio_l_out(10 downto 4)  & '0'
                & s_audio_l_out(3 downto 0)   & s_audio_l_in(23 downto 21)    & '0'
                & s_audio_l_in(20 downto 14)  & '0'
                & s_audio_l_in(13 downto 7)   & '0'
                & s_audio_l_in(6 downto 0)    & '0' 
				& '0' & s_audio_r_out(23 downto 18) & '0'
                & s_audio_r_out(17 downto 11) & '0'
                & s_audio_r_out(10 downto 4)  & '0'
                & s_audio_r_out(3 downto 0)   &  s_audio_r_in(23 downto 21)    & '0'
                & s_audio_r_in(20 downto 14)  & '0'
                & s_audio_r_in(13 downto 7)   & '0'
                & s_audio_r_in(6 downto 0)    & '0';
			fsm <= s_send;
		end if;
	elsif fsm = s_send then
		if FT232R_ready = '1' then
			FT232R_byte_snd <= packet((13-packet_bytecounter)*8+7 downto (13-packet_bytecounter)*8);
			FT232R_start_snd <= '1';
			if packet_bytecounter < 13 then
				packet_bytecounter <= packet_bytecounter +1;
			else
				fsm <= s_wait;
			end if;
		end if;
	end if;
end process;

process begin
	wait until rising_edge(s_clk_8_192_MHz_MCLK);
	clkdiv <= clkdiv +1;
end process;
s_clk_2_048_MHz_SCLK <= clkdiv(1);
s_clk_32_kHz_LRCK <= clkdiv(7);

--*************************alle PMODS zuweisen********************************************
DA_LRCK <= s_clk_32_kHz_LRCK;
DA_MCLK <= s_clk_8_192_MHz_MCLK;
DA_SCLK <= s_clk_2_048_MHz_SCLK;
------------------------------------
AD_LRCK <= s_clk_32_kHz_LRCK;
AD_MCLK <= s_clk_8_192_MHz_MCLK;
AD_SCLK <= s_clk_2_048_MHz_SCLK;
        
DA_SDIN <= s_DA_SDIN;
--****************************************************************************************

Filter_L: FIR_L port map(
	aclk => s_clk_8_192_MHz_MCLK,
	s_axis_data_tvalid => s_new_sample,
	s_axis_data_tready => open,
	s_axis_data_tdata => s_audio_l_out,
	m_axis_data_tvalid => open,
	m_axis_data_tdata => s_audio_l_in);
	
Filter_R: FIR_R port map(
	aclk => s_clk_8_192_MHz_MCLK,
	s_axis_data_tvalid => s_new_sample,
	s_axis_data_tready => open,
	s_axis_data_tdata => s_audio_r_out,
	m_axis_data_tvalid => open,
	m_axis_data_tdata => s_audio_r_in);
	
MMCM: CLK_I2S port map(
	clk200 => s_clk_200_MHz,
	clk8_192 => s_clk_8_192_MHz_MCLK,
	clk_in1_p => sysclk_p,
	clk_in1_n => sysclk_n);
  
i2s_interface : i2s_data_interface port map (
	clk           => s_clk_8_192_MHz_MCLK,
	audio_l_in    => s_audio_l_in,  -- std_logic_vector als Eingangs fr Umwandlung als serielle Ausgabe
	audio_r_in    => s_audio_r_in,  -- std_logic_vector als Eingangs fr Umwandlung als serielle Ausgabe
	audio_l_out   => s_audio_l_out, -- umgewandelte Eingangsdaten
	audio_r_out   => s_audio_r_out, -- umgewandelte Eingangsdaten
	new_sample    => s_new_sample,
	i2s_bclk      => s_clk_2_048_MHz_SCLK,
	i2s_d_out     => s_DA_SDIN,
	i2s_d_in      => AD_SDOUT,
	i2s_lr        => s_clk_32_kHz_LRCK);
	
UART_FT232R: UART
    Generic map(
	clk_freq => 200000000,
	baudrate => 8000000)
    Port map(
	clk => s_clk_200_MHz,
	start_snd => FT232R_start_snd,
	start_rcv => FT232R_start_rcv,
	byte_snd => FT232R_byte_snd,
	byte_rcv => FT232R_byte_rcv,
	tx => FT232R_TX,
	rx => FT232R_RX,
	ready => FT232R_ready);

end Behavioral;