library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity UART is
    Generic(
	clk_freq: integer;
	baudrate: integer);
    Port(
	clk: in STD_LOGIC;
	start_snd: in std_logic;
	start_rcv: out std_logic;
	byte_snd : in std_logic_vector(7 downto 0);
	byte_rcv : out std_logic_vector(7 downto 0);
	tx : out std_logic;
	rx : in std_logic;
	ready: out STD_LOGIC);
end UART;

architecture Behavioral of UART is
constant txbits : integer := 12;
constant txclocksperbit: integer := clk_freq/baudrate;

signal txsr     : std_logic_vector(txbits-1 downto 0):=(others => '1');  -- Startbit, 8 Datenbits, Stopbit
signal txbitcnt : integer range 0 to txbits:= txbits;
signal txcnt    : integer range 0 to txclocksperbit-1:= 0;

signal rxd_sr   : std_logic_vector (3 downto 0) := "1111";         -- Flankenerkennung und Eintakten
signal rxsr     : std_logic_vector (7 downto 0) := "00000000";     -- 8 Datenbits
signal rxbitcnt : integer range 0 to 9 := 9;
signal rxcnt    : integer range 0 to (clk_freq/baudrate)-1 := 0;
signal rxbusy   : std_logic := '0';

begin
   -- Senden
   process begin
      wait until rising_edge(clk);
      if start_snd = '1' and txbitcnt = txbits then -- steigende Flanke, los gehts
         txcnt    <= 0;                      -- Zähler initialisieren
         txbitcnt <= 0;                      
         txsr     <= "111" & byte_snd & '0';    -- Stopbit, 8 Datenbits, Startbit, rechts gehts los
      else
         if txcnt < txclocksperbit-1 then
            txcnt <= txcnt+1;
         else  -- nächstes Bit ausgeben  
            if txbitcnt < txbits then
              txcnt    <= 0;
              txbitcnt <= txbitcnt+1;
              txsr     <= '1' & txsr(txsr'left downto 1);
            end if;
         end if;
      end if;
   end process;
   tx     <= txsr(0);  -- LSB first
   ready <= '0' when start_snd = '1' or txbitcnt < txbits else '1';
   
   -- Empfangen
   process begin
      wait until rising_edge(clk);
      rxd_sr <= rxd_sr(rxd_sr'left-1 downto 0) & rx;
      if (rxbitcnt<9) then    -- Empfang läuft
         if(rxcnt<(clk_freq/baudrate)-1) then 
            rxcnt    <= rxcnt+1;
         else
            rxcnt    <= 0; 
            rxbitcnt <= rxbitcnt+1;
            rxsr     <= rxd_sr(rxd_sr'left-1) & rxsr(rxsr'left downto 1); -- rechts schieben, weil LSB first
         end if;
      else -- warten auf Startbit
         if (rxd_sr(3 downto 2) = "10") then                 -- fallende Flanke Startbit
            rxcnt    <= ((clk_freq/baudrate)-1)/2; -- erst mal nur halbe Bitzeit abwarten
            rxbitcnt <= 0;
         end if;
      end if;
	  start_rcv <= '0';
	  if rxbitcnt<9 then
		rxbusy <= '1';
	  else
		rxbusy <= '0';
		if rxbusy = '1' then
		  start_rcv <= '1';
		end if;
	  end if;
   end process;
   byte_rcv <= rxsr;

end Behavioral;